from django.conf.urls import url

from . import views


urlpatterns = [
    url(r'^md-upload', views.upload_file, name='dropzone_upload'),
    url(r'^md-delete', views.delete_file, name='dropzone_delete'),
    url(r'^md-filelist', views.file_list, name='dropzone_filelist'),
    url(r'^md-history', views.history_list, name='markdown_history'),
]


from django.conf import settings
from django.forms import widgets
from django.forms.utils import flatatt
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from markdownme.settings import MARKDOWNME_DEFAULT_CONFIG


class MarkdownmeWidget(widgets.Textarea):
    template_name = 'markdownme/textedit.html'

    class Media():
        css = {
            'all': (
                'markdownme/css/dropzone.min.css',
                'markdownme/css/markdownme.min.css',
                'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css',
                )
            }
        js = (
            'markdownme/js/dropzone.min.js',
            'markdownme/js/markdownme.min.js',
            'markdownme/js/diff.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/showdown/1.6.4/showdown.min.js',
            )

    def render(self, name, value, attrs=[]):
        if value is None:
            value = ''
        
        config = getattr(settings, 'MARKDOWNME_CONFIG', MARKDOWNME_DEFAULT_CONFIG)
        file_upload_allowed = config.get('FILE_UPLOAD', MARKDOWNME_DEFAULT_CONFIG['FILE_UPLOAD'])
        upload_root = config.get('UPLOAD_ROOT', MARKDOWNME_DEFAULT_CONFIG['UPLOAD_ROOT'])
        if len(upload_root) > 0 and upload_root[-1:] != '/':
            upload_root = upload_root + '/'
        allowed_file_types = config.get('ALLOWED_FILETYPES', MARKDOWNME_DEFAULT_CONFIG['ALLOWED_FILETYPES'])
        allowed_file_types = ','.join(allowed_file_types)
        history_allowed = config.get('HISTORY', MARKDOWNME_DEFAULT_CONFIG['HISTORY'])
        iframe_preview = config.get('IFRAME_PREVIEW', MARKDOWNME_DEFAULT_CONFIG['IFRAME_PREVIEW'])
        safe_preview = config.get('SAFE_PREVIEW', MARKDOWNME_DEFAULT_CONFIG['SAFE_PREVIEW'])
        two_pane_history = config.get('TWO_PANE_HISTORY', MARKDOWNME_DEFAULT_CONFIG['TWO_PANE_HISTORY'])
        max_filesize = config.get('MAX_FILE_SIZE', MARKDOWNME_DEFAULT_CONFIG['MAX_FILE_SIZE'])
        
        attrs['name'] = name
        context = {
            'attrs': flatatt(self.build_attrs(attrs)),
            'value': value,
            'file_upload_allowed': file_upload_allowed,
            'markdownme_media_root': settings.MEDIA_URL + upload_root,
            'allowed_file_types': allowed_file_types,
            'history_allowed': history_allowed,
            'safe_preview': safe_preview,
            'iframe_preview': iframe_preview,
            'two_pane_history': two_pane_history,
            'max_filesize': max_filesize,
        }
        return mark_safe(render_to_string(self.template_name, context))


from __future__ import unicode_literals

import os
import uuid

from django.conf import settings
from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch import receiver
import markdown

from markdownme.settings import MARKDOWNME_DEFAULT_CONFIG

from .fields import MarkdownmeTextField, MarkdownmeIdField

config = getattr(settings, 'MARKDOWNME_CONFIG', MARKDOWNME_DEFAULT_CONFIG)
hash_filenames = config.get('HASH_FILENAMES', MARKDOWNME_DEFAULT_CONFIG['HASH_FILENAMES'])
if hash_filenames:
    import hashlib

class MarkdownmeHistory(models.Model):
    class Meta:
        verbose_name = 'Markdownme History Entry'
        verbose_name_plural = 'Markdownme History Entries'
        
    markdown_text = models.TextField(default = None, null = True, blank = True)
    markdown_identifier = models.UUIDField(editable = False, blank = True, null = True)
    date = models.DateTimeField(auto_now_add = True)

class MarkdownmeEntry(models.Model):
    markdown_text = MarkdownmeTextField(default = None, blank = True, null = True)
    parsed_text = models.TextField(editable = False, default = None, null = True)
    markdown_identifier = MarkdownmeIdField(default = uuid.uuid4, max_length = 32, unique = True)
    
    class Meta:
        abstract = True
    
    def save(self, *args, **kwargs):
        if (self.markdown_text != None):
            
            sanitize = config.get('SANITIZE_MARKDOWN', MARKDOWNME_DEFAULT_CONFIG['SANITIZE_MARKDOWN'])
            self.parsed_text = markdown.markdown(self.markdown_text, output_format = 'html5')
            if sanitize:
                try:
                    import bleach
                except ImportError:
                    print("SANITIZE_MARKDOWN is set to True, but Bleach is not installed! This input will not be saved")
                    return
                allowed_tags = config.get('ALLOWED_TAGS', MARKDOWNME_DEFAULT_CONFIG['ALLOWED_TAGS'])
                allowed_styles = config.get('ALLOWED_STYLES', MARKDOWNME_DEFAULT_CONFIG['ALLOWED_STYLES'])
                allowed_attributes = config.get('ALLOWED_ATTRIBUTES', MARKDOWNME_DEFAULT_CONFIG['ALLOWED_ATTRIBUTES'])
                allowed_protocols = config.get('ALLOWED_PROTOCOLS', MARKDOWNME_DEFAULT_CONFIG['ALLOWED_PROTOCOLS'])
                self.parsed_text = bleach.clean(self.parsed_text, tags = allowed_tags, styles = allowed_styles,
                                                attributes = allowed_attributes, protocols = allowed_protocols)
            MarkdownmeHistory.objects.create(markdown_text = self.markdown_text, markdown_identifier = self.markdown_identifier)
            super(MarkdownmeEntry, self).save(*args, **kwargs)

def get_upload_to(instance, filename):
    upload_root = config.get('UPLOAD_ROOT', MARKDOWNME_DEFAULT_CONFIG['UPLOAD_ROOT'])
    if hash_filenames:
        name, ext = os.path.splitext(filename)
        md5 = hashlib.md5()
        md5.update(name.encode('utf-8'))
        name = md5.hexdigest() + ext
        return os.path.join(upload_root, instance.markdown_identifier, name)
    else:
        return os.path.join(upload_root, instance.markdown_identifier, filename)

class MarkdownmeMedia(models.Model):
    class Meta:
        verbose_name = 'Markdownme Media File'

    media_file = models.FileField(upload_to = get_upload_to, max_length = 100)   
    markdown_identifier = models.UUIDField(editable = False, blank = True, null = True)
    
    def filename(self):
        return os.path.basename(self.media_file.name)
    
    def __str__(self):
        return self.filename()

@receiver(pre_delete, sender = MarkdownmeMedia)
def delete_media_file(instance, **kwargs):
    instance.media_file.delete()
    upload_root = config.get('UPLOAD_ROOT', MARKDOWNME_DEFAULT_CONFIG['UPLOAD_ROOT'])
    mediaDirectory = os.path.join(settings.MEDIA_ROOT, upload_root, str(instance.markdown_identifier).replace('-', ''))
    try:
        if not os.listdir(mediaDirectory):
            os.rmdir(mediaDirectory)
    except FileNotFoundError:
        print("There was a problem removing a seemingly empty folder probably caused by inconsistency of Markdownme settings and media file data in database")
    
#called for every model delete in the app!
#basically solves the cascading deletion + removal of empty media folder left
#might be a performance issue in a big architecture
#specifying an abstract class as a sender doesnt trigger the signal and this seems to be the only simple solution
#for subclass in MarkdownmeEntry.__subclasses__():
    #pre_delete.connect(delete_markdown_instance, subclass)
@receiver(pre_delete)
def delete_markdown_instance(instance, sender, **kwargs):
    if not issubclass(sender, MarkdownmeEntry):
        return
    
    for media_file in MarkdownmeMedia.objects.filter(markdown_identifier = instance.markdown_identifier):
        media_file.delete()
    
    for history_entry in MarkdownmeHistory.objects.filter(markdown_identifier = instance.markdown_identifier):
        history_entry.delete()
        
    os.rmdir('%s/%s' % (settings.MEDIA_ROOT, str(instance.markdown_identifier).replace('-', '')))


import os.path

from django.conf import settings
from django.conf import settings
from django.http import HttpResponse
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt

from markdownme.settings import MARKDOWNME_DEFAULT_CONFIG

from .models import MarkdownmeMedia, MarkdownmeHistory


config = getattr(settings, 'MARKDOWNME_CONFIG', MARKDOWNME_DEFAULT_CONFIG)
csrf_protected = config.get('CSRF_PROTECTED', MARKDOWNME_DEFAULT_CONFIG['SANITIZE_MARKDOWN'])
if csrf_protected:
    def upload_file(request):
        if request.method == 'POST':
            markdown_identifier = request.POST.get('markdown_identifier', None)
            if markdown_identifier is None:
                return HttpResponse(status=404)
            
            createdFile = MarkdownmeMedia.objects.create(media_file = request.FILES['file'], markdown_identifier = markdown_identifier)
            json_data = dict(url = createdFile.media_file.url)
            return JsonResponse(json_data)
            
        return HttpResponse(status=404)

    def delete_file(request):
        if request.method == 'POST':
            config = getattr(settings, 'MARKDOWNME_CONFIG', MARKDOWNME_DEFAULT_CONFIG)
            upload_root = config.get('UPLOAD_ROOT', MARKDOWNME_DEFAULT_CONFIG['UPLOAD_ROOT'])
            filename = request.POST.get('filename', '')
            markdown_identifier = request.POST.get('markdown_identifier', None)
            fileToDelete = get_object_or_404(MarkdownmeMedia, media_file = os.path.join(upload_root, markdown_identifier, filename), markdown_identifier = markdown_identifier)
            fileToDelete.delete()
            return HttpResponse(status=200)
            
        return HttpResponse(status=404)
else:
    @csrf_exempt
    def upload_file(request):
        if request.method == 'POST':
            markdown_identifier = request.POST.get('markdown_identifier', None)
            if markdown_identifier is None:
                return HttpResponse(status=404)
            
            createdFile = MarkdownmeMedia.objects.create(media_file = request.FILES['file'], markdown_identifier = markdown_identifier)
            json_data = dict(url = createdFile.media_file.url)
            return JsonResponse(json_data)
            
        return HttpResponse(status=404)

    @csrf_exempt
    def delete_file(request):
        if request.method == 'POST':
            config = getattr(settings, 'MARKDOWNME_CONFIG', MARKDOWNME_DEFAULT_CONFIG)
            upload_root = config.get('UPLOAD_ROOT', MARKDOWNME_DEFAULT_CONFIG['UPLOAD_ROOT'])
            filename = request.POST.get('filename', '')
            markdown_identifier = request.POST.get('markdown_identifier', None)
            fileToDelete = get_object_or_404(MarkdownmeMedia, media_file = os.path.join(upload_root, markdown_identifier, filename), markdown_identifier = markdown_identifier)
            fileToDelete.delete()
            return HttpResponse(status=200)


def file_list(request):
    if request.method == 'GET':
        markdown_identifier = request.GET.get('markdown_identifier', None)
        data = []
        for file in MarkdownmeMedia.objects.filter(markdown_identifier = markdown_identifier):
            data.append(dict(url = file.media_file.url, size = file.media_file.size))
        json_data = dict(files = data)
        return JsonResponse(json_data)
        
    return HttpResponse(status=404)

def history_list(request):
    if request.method == 'GET':
        markdown_identifier = request.GET.get('markdown_identifier', None)
        data = []
        for history_entry in MarkdownmeHistory.objects.filter(markdown_identifier = markdown_identifier).order_by('-date'):
            data.append(dict(text = history_entry.markdown_text, date = history_entry.date.strftime('%b %d, %Y %H:%M')))
        json_data = dict(entries = data)
        return JsonResponse(json_data)
        
    return HttpResponse(status=404)


from django.db import models
from django.forms.widgets import HiddenInput

from .widgets import MarkdownmeWidget


class MarkdownmeTextField(models.TextField):
    def formfield(self, **kwargs):
        kwargs['widget'] = MarkdownmeWidget
        return super(MarkdownmeTextField, self).formfield(**kwargs)

class MarkdownmeIdField(models.UUIDField):
    def formfield(self, **kwargs):
        kwargs['widget'] = HiddenInput
        return super(MarkdownmeIdField, self).formfield(**kwargs)



#from distutils.core import setup
#setup(
  #name = 'django-markdownme',
  #packages = ['django-markdownme'], # this must be the same as the name above
  #version = '0.1',
  #description = 'Text editor component with Markdown syntax support, file upload manager and history of changes.',
  #author = 'Oliver Kovac',
  #author_email = 'oliver.kovac.2@student.tuke.sk',
  #url = 'https://github.com/peterldowns/mypackage', # use the URL to the github repo
  #download_url = 'https://github.com/peterldowns/mypackage/archive/0.1.tar.gz', # I'll explain this in a second
  #keywords = ['Django', 'Markdown', 'file upload', 'history', 'preview',], # arbitrary keywords
  #classifiers = [],
#)

import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-markdownme',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',  # example license
    description='A Django text editor with Markdown and file upload support',
    long_description=README,
    url='https://github.com/oliverkovac/django-markdownme',
    author = 'Oliver Kovac',
    author_email = 'oliver.kovac.2@student.tuke.sk',
    install_requires=[
        'markdown',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


#####################################################################
# This is a default config for the app
#
# Use this structure in your project settings, but name it
# MARKDOWNME_CONFIG
# Anything not specified there will be used from this one
#
# DO NOT MODIFY (UNLESS YOU REALLY KNOW WHAT YOU'RE DOING)
#####################################################################

MARKDOWNME_DEFAULT_CONFIG = {
    'FILE_UPLOAD': True,
    'UPLOAD_ROOT': 'markdownme',
    'ALLOWED_FILETYPES': ['image/*', 'application/pdf', 'application/x-rar-compressed', 'application/x-gtar', 'application/x-tar', 'application/x-7z-compressed', 'application/zip'],
    'HISTORY': True,
    'SANITIZE_MARKDOWN': True,
    'IFRAME_PREVIEW': True,
    'SAFE_PREVIEW': False,
    'TWO_PANE_HISTORY': True,
    'CSRF_PROTECTED': True,
    'HASH_FILENAMES': False,
    'MAX_FILE_SIZE': 5,
    'ALLOWED_TAGS': [
        'a', 'abbr', 'acronym', 'b', 'blockquote', 'br', 'button', 'caption', 'cite',
        'code', 'col', 'colgroup', 'dd', 'del', 'div', 'dl', 'dt', 'em', 'figure',
        'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'header', 'hr', 'i', 'img', 'ins',
        'li', 'multicol', 'ol', 'p', 'pre', 'q', 's', 'samp', 'section', 'small',
        'source', 'span', 'strike', 'strong', 'sub', 'sup', 'table', 'tbody', 'td',
        'tfoot', 'th', 'thead', 'time', 'tr', 'u', 'ul', 'var', 'video',
    ],
    'ALLOWED_STYLES': [
        'background-color', 'color', 'height', 'volume', 'width',
    ],
    'ALLOWED_ATTRIBUTES': {
        u'a': [u'href', u'title'],
        u'acronym': [u'title'],
        u'abbr': [u'title'],
        'h1': ['style', ],
        'h2': ['style', ],
        'h3': ['style', ],
        'h4': ['style', ],
        'h5': ['style', ],
        'h6': ['style', ],
        'span': ['style', ],
        'img': ['src', 'id', 'align', 'alt', 'class', 'is', 'title', 'style', ],
        'audio': ['src', 'type', 'controls', ], 
        'video': ['src', 'type', 'controls', 'width', 'height', 'style', ],
        'source': ['src', 'type', 'media', ],
        },
    'ALLOWED_PROTOCOLS': [u'http', u'https', u'mailto'],        
}



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-05-23 19:03
from __future__ import unicode_literals

from django.db import migrations, models
import markdownme.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MarkdownmeHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('markdown_text', models.TextField(blank=True, default=None, null=True)),
                ('markdown_identifier', models.UUIDField(blank=True, editable=False, null=True)),
                ('date', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name_plural': 'Markdownme History Entries',
                'verbose_name': 'Markdownme History Entry',
            },
        ),
        migrations.CreateModel(
            name='MarkdownmeMedia',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('media_file', models.FileField(upload_to=markdownme.models.get_upload_to)),
                ('markdown_identifier', models.UUIDField(blank=True, editable=False, null=True)),
            ],
            options={
                'verbose_name': 'Markdownme Media File',
            },
        ),
    ]



from django.contrib import admin

from .models import MarkdownmeMedia, MarkdownmeHistory


# Register your models here.
class MarkdownHistoryAdmin(admin.ModelAdmin):
    list_display = ('markdown_identifier', 'date')

admin.site.register(MarkdownmeMedia)
admin.site.register(MarkdownmeHistory, MarkdownHistoryAdmin)